<?php

namespace Drupal\term_glossary\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

class TermGlossaryManager {

  /**
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected ConfigManagerInterface $configManager;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $termStorage;

  /**
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected CacheBackendInterface $cache;

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  protected string $cacheId = 'glossary_terms_array';

  protected array $termList;

  public function __construct(ConfigManagerInterface $configManager, EntityTypeManagerInterface $entityTypeManager, CacheBackendInterface $cache) {
    $this->configManager = $configManager;
    $this->entityTypeManager = $entityTypeManager;
    $this->cache = $cache;
    $this->termStorage = $this->entityTypeManager->getStorage('taxonomy_term');
    $this->config = $configManager->getConfigFactory()->get('term_glossary.glossaryconfig');
  }

  /**
   * @param $text
   *   Original string.
   *
   * @return string
   *   String with highlighted terms.
   */
  public function replaceFieldValue($text, $vocabulary = NULL) {
    $this->termList = $vocabulary ? $this->getTerms($vocabulary) : $this->termList = $this->getTerms();
    $new_value = '';
    if (!empty($text)) {
      $new_value = $text;
      foreach ($this->termList as $term_id => $term_array) {
        $name = preg_quote($term_array['name'], '/');
        preg_match_all("/$name+/i", $new_value, $matches);
        if (is_array($matches[0]) && count($matches[0]) >= 1) {
          foreach ($matches[0] as $match) {
            $new_value = str_replace($match, '<span class="glos-term" data-gterm="' . $term_id . '">'.$match.'</span>', $new_value);
          }
        }
      }
    }

    return $new_value;
  }

  /**
   * Get configuration.
   *
   * @param string $key
   *   Config key.
   *
   * @return string
   *   Config value.
   */
  public function getConfigValue(string $key) :string {
    return $this->config->get($key);
  }

  /**
   * Get an updated list of glossary terms.
   *
   * @return array
   *   Glossary term list
   */
  private function getTerms($vocabulary = NULL) {
    $vocab = $vocabulary ? $vocabulary : $this->config->get('vocab');
    $cacheId = $this->cacheId . '-' . $vocab;
    $data = $this->cache->get($cacheId);
    if (empty($data)) {
      $terms = $this->updateTermList($vocab);
    }
    else {
      $query = $this->termStorage->getQuery();
      $query->condition('vid', $vocab);
      $entity_ids = $query->execute();
      // Someone could have added new term since last cache.
      if (count($entity_ids) == count($data->data)) {
        // Get the list from cache.
        $terms = $data->data;
      }
      else {
        // Set cache again.
        $terms = $this->updateTermList($vocab);
      }

    }
    return $terms;
  }

  /**
   * Update term list.
   *
   * @param $vocab
   *   Glossary vocabulary id.
   *
   * @return array
   *   Glossary term list
   */
  private function updateTermList($vocab) {
    $terms = $this->termStorage->loadByProperties(['vid' => $vocab]);
    $terms_array = [];
    /**
     * @var  integer $tid
     * @var \Drupal\taxonomy\Entity\Term $term
     */
    foreach ($terms as $tid => $term) {
      $terms_array[$tid] = [
        'tid' => $tid,
        'name' => $term->getName(),
      ];
    }
    $cacheId = $this->cacheId . '-' . $vocab;
    $this->cache->set($cacheId, $terms_array, CacheBackendInterface::CACHE_PERMANENT, []);
    return $terms_array;
  }
}
